using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Blockmania.Classes;




namespace Blockmania
{

    public class BlockManiaGame : Microsoft.Xna.Framework.Game
    {
        // *******************Mngu muutujad *******************

        // Default XNA muutujad
        private GraphicsDeviceManager graphics;
        private SpriteBatch spriteBatch;

        // game font.
        private SpriteFont gameFont;

        // Muutujad lugemiseks klaviatuurilt
        private KeyboardState keyStatePrevious = Keyboard.GetState();
        private KeyboardState keyStateCurrent = Keyboard.GetState();
        private GamePadState padStatePrevious = GamePad.GetState(PlayerIndex.One);
        private GamePadState padStateCurrent = GamePad.GetState(PlayerIndex.One);

        // Texture
        private Texture2D titleScreen;
        private Texture2D gameScreen;
        private Texture2D pauseScreen;
        private Texture2D gameover;
        private static Texture2D[] tileTextures;

        // Klots object
        private Klots Klots;

        // Laud object
        private Laud Laud;

        // Mootor object
        private Mootor Mootor;


        #region Properties

        public static Texture2D[] TileTextures
        {
            get { return tileTextures; }

        }


        #endregion


        // ******************* XNA Meetodid *******************

        // Constructor
        public BlockManiaGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Stime mnguakna suuruse ja pealkirja
            graphics.PreferredBackBufferWidth = Mootor.ekraani_laius;
            graphics.PreferredBackBufferHeight = Mootor.ekraani_korgus;
            Window.Title = "BlockMania";
        }


        protected override void Initialize()
        {
            // Initsialiseerime klotsi, mngulaua ja mootori
            Klots = new Klots();
            Laud = new Laud();
            Mootor = new Mootor();

            base.Initialize();
        }


        protected override void LoadContent()
        {
            // Teeme uue SpriteBatchi.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Loadime fondi
            gameFont = Content.Load<SpriteFont>("Pildid/GameFont");

            // Loadime men ja mngu tausta
            titleScreen = Content.Load<Texture2D>("Pildid/Ekraan");
            gameScreen = Content.Load<Texture2D>("Pildid/Taust");
            gameover = Content.Load<Texture2D>("Pildid/gameover");

            // Loadime pausimen tausta
            pauseScreen = Content.Load<Texture2D>("Pildid/pause");

            // Loadime klotsi tiled
            tileTextures = new Texture2D[7];
            tileTextures[0] = Content.Load<Texture2D>("Pildid/Lilla");
            tileTextures[1] = Content.Load<Texture2D>("Pildid/Hall");
            tileTextures[2] = Content.Load<Texture2D>("Pildid/Kollane");
            tileTextures[3] = Content.Load<Texture2D>("Pildid/Oraanz");
            tileTextures[4] = Content.Load<Texture2D>("Pildid/Punane");
            tileTextures[5] = Content.Load<Texture2D>("Pildid/Roheline");
            tileTextures[6] = Content.Load<Texture2D>("Pildid/Sinine");
        }


        protected override void UnloadContent()
        {

        }


        protected override void Update(GameTime gameTime)
        {
            keyStateCurrent = Keyboard.GetState();
            padStateCurrent = GamePad.GetState(PlayerIndex.One);

            // Lahkume rakendusest kui vajutatakse "esc"
            if ((keyStateCurrent.IsKeyDown(Keys.Escape)) || (padStateCurrent.IsButtonDown(Buttons.Back)))
                this.Exit();

            //Kontrolli "P" vajutust.
            Mootor.CheckPausePlay(keyStateCurrent, keyStatePrevious,padStateCurrent,padStatePrevious, Laud, Klots);


            if (!Mootor.GameOver && !Mootor.GamePause)
            {
                // Mnguloogika
                Mootor.UpdateTimer(gameTime, Laud, Klots);
                Mootor.ProcessKeyboard(keyStateCurrent, keyStatePrevious, Laud, Klots);
                Mootor.xboxcontroller(padStateCurrent, padStatePrevious, Laud, Klots);
                Mootor.ManguLoogika(Laud);
            }

            keyStatePrevious = keyStateCurrent;
            padStatePrevious = padStateCurrent;

            base.Update(gameTime);
        }


        protected override void Draw(GameTime gameTime)
        {
            if (!Mootor.GameOver)
            {
                // Joonistab mnguajal ekraanile
                Mootor.JoonistaManguEkraan(gameScreen, spriteBatch);
                Mootor.JoonistaKoikKlotsid(Laud, Klots, spriteBatch, gameFont);
                Mootor.JoonistaSkoor(gameFont, spriteBatch);

            }
            else
            {
                // Joonistab men
                Mootor.JoonistaManguEkraan(titleScreen, spriteBatch);

            }
            if (Mootor.GamePause == true)
            {
                // Joonistan mngu pausile pannes ekraanile eraldi pildi
                Mootor.JoonistaManguEkraan(pauseScreen, spriteBatch);

            }
            if (Mootor.GameOver == true && Mootor.Newgame == false)
            {
                //Kui mng lbi joonista gameover
                Mootor.JoonistaManguEkraan(gameover, spriteBatch);
                Mootor.JoonistaTulemus(gameFont, spriteBatch);
            }

            base.Draw(gameTime);
        }
    }

}